# -*- coding: utf-8 -*-

__version__ = "0.4.0"
__author__ = "Matthew Love"
__credits__ = "CIRES"

# Import everything except the individual modules.
from . import fred
from . import core
from . import cli
from . import spatial
from . import registry

